"""
Analysis specific job handler for B -> D(*(*)) tau nu analysis.

Performs run time sampling and job submission for specified
configuration files.

Felix Metzner 2018
"""

import os
import b2luigi as luigi
import analysistools.batch_tools.job_handler as handler
import analysistools.batch_tools.tasks as analysis_tasks

from rdstar.batch_jobs.check_configs import config_check

production_tag = "prod_02ndMay2021"
config_check(production_tag=production_tag)

config_base_dir = "/home/belle2/rvantond/felix/RDStar/rdstar/configs/"

# config_production_dir = f"{production_tag}_stream2"
config_production_dir = f"{production_tag}_stream7"

batch_configs = [
    # os.path.join(config_base_dir, config_production_dir, "batch_config.json"),  # Test configuration
    os.path.join(config_base_dir, config_production_dir, "charged_batch_config.json"),
    os.path.join(config_base_dir, config_production_dir, "mixed_batch_config.json"),
    # os.path.join(config_base_dir, config_production_dir, "uds_batch_config.json"),
    # os.path.join(config_base_dir, config_production_dir, "charm_batch_config.json"),
    # os.path.join(config_base_dir, config_production_dir, "off_resonance_mc_batch_config.json"),
]

# Using charged batch config as base for run time sampling:
rts_batch_config = os.path.join(config_base_dir, config_production_dir, "charged_batch_config.json")


class JobHandlerTask(luigi.WrapperTask):
    def get_task_file_dir(self):
        batch_config = handler.load_json_config(batch_configs[0], hint="Loading batch config in JobHandlerTask")
        global_config = handler.load_json_config(
            batch_config["global_config"],
            hint="Loading global config in JobHandlerTask",
        )
        os.makedirs(global_config["target_directory"], exist_ok=True)
        return os.path.join(global_config["target_directory"], f"{self.get_task_family()}_{config_production_dir}")

    def get_log_file_dir(self):
        return self.get_task_file_dir()

    def requires(self):
        # yield handler.RunTimeSamplingMaster(batch_config_file=rts_batch_config, sample_chunks=10, chunk_size=5000)

        for batch_config in batch_configs:
            yield analysis_tasks.Master(batch_config_file=batch_config)


if __name__ == "__main__":
    num_workers = 10000
    print(f"Starting to process events with\n\t\tworkers = {num_workers}\n\t\tbatch configuration files:")
    for config in batch_configs:
        print(config)

    luigi.process(JobHandlerTask(), workers=num_workers)
