"""
Providing latex strings for particles.

Felix Metzner 2021
"""

from rdstar.latex_labels.latex_string_methods import LatexObsNameMethods


__all__ = [
    "LatexParticleNames",
]


class LatexParticleNames:
    pi_str = r"$\pi$"  # type: str
    pi_zero_str = LatexObsNameMethods.add_super_script(var_str=pi_str, script=r"$0$")  # type: str
    pi_pm_str = LatexObsNameMethods.add_super_script(var_str=pi_str, script=r"$\pm$")  # type: str
    pi_plus_str = LatexObsNameMethods.add_super_script(var_str=pi_str, script=r"$+$")  # type: str
    pi_minus_str = LatexObsNameMethods.add_super_script(var_str=pi_str, script=r"$-$")  # type: str

    k_str = r"$\mathrm{K}$"  # type: str
    k_pm_str = LatexObsNameMethods.add_super_script(var_str=k_str, script=r"$\pm$")  # type: str
    k_plus_str = LatexObsNameMethods.add_super_script(var_str=k_str, script=r"$+$")  # type: str
    k_minus_str = LatexObsNameMethods.add_super_script(var_str=k_str, script=r"$-$")  # type: str
    k_zero_str = LatexObsNameMethods.add_super_script(var_str=k_str, script=r"$0$")  # type: str
    k_short_str = LatexObsNameMethods.add_sub_script(var_str=k_zero_str, script=r"$\mathrm{Short}$")  # type: str
    k_long_str = LatexObsNameMethods.add_sub_script(var_str=k_zero_str, script=r"$\mathrm{Long}$")  # type: str

    gamma_str = r"$\gamma$"  # type: str
    gamma_gamma_str = gamma_str[:-1] + gamma_str[1:]  # type: str
    virtual_gamma_str = LatexObsNameMethods.add_super_script(var_str=gamma_str, script=r"$\mathrm{Virtual}$")  # type: str

    electron_str = r"$\mathrm{e}$"  # type: str
    electron_pm_str = LatexObsNameMethods.add_super_script(var_str=electron_str, script=r"$\pm$")  # type: str
    electron_plus_str = LatexObsNameMethods.add_super_script(var_str=electron_str, script=r"$+$")  # type: str
    electron_minus_str = LatexObsNameMethods.add_super_script(var_str=electron_str, script=r"$-$")  # type: str
    mu_str = r"$\mu$"  # type: str
    muon_pm_str = LatexObsNameMethods.add_super_script(var_str=mu_str, script=r"$\pm$")  # type: str
    muon_plus_str = LatexObsNameMethods.add_super_script(var_str=mu_str, script=r"$+$")  # type: str
    muon_minus_str = LatexObsNameMethods.add_super_script(var_str=mu_str, script=r"$-$")  # type: str
    tau_str = r"$\tau$"  # type: str
    tau_pm_str = LatexObsNameMethods.add_super_script(var_str=tau_str, script=r"$\pm$")  # type: str
    tau_plus_str = LatexObsNameMethods.add_super_script(var_str=tau_str, script=r"$+$")  # type: str
    tau_minus_str = LatexObsNameMethods.add_super_script(var_str=tau_str, script=r"$-$")  # type: str
    lepton_str = r"$\ell$"  # type: str
    lepton_pm_str = LatexObsNameMethods.add_super_script(var_str=lepton_str, script=r"$\pm$")  # type: str
    lepton_plus_str = LatexObsNameMethods.add_super_script(var_str=lepton_str, script=r"$+$")  # type: str
    lepton_minus_str = LatexObsNameMethods.add_super_script(var_str=lepton_str, script=r"$-$")  # type: str

    nu_str = r"$\nu$"  # type: str
    nu_lep_str = LatexObsNameMethods.add_sub_script(var_str=nu_str, script=lepton_str)  # type: str
    nu_e_str = LatexObsNameMethods.add_sub_script(var_str=nu_str, script=electron_str)  # type: str
    nu_mu_str = LatexObsNameMethods.add_sub_script(var_str=nu_str, script=mu_str)  # type: str
    nu_tau_str = LatexObsNameMethods.add_sub_script(var_str=nu_str, script=tau_str)  # type: str
    anti_nu_str = LatexObsNameMethods.get_anti_particle(particle_str=nu_str)  # type: str
    anti_nu_lep_str = LatexObsNameMethods.get_anti_particle(particle_str=nu_lep_str)  # type: str
    anti_nu_e_str = LatexObsNameMethods.get_anti_particle(particle_str=nu_e_str)  # type: str
    anti_nu_mu_str = LatexObsNameMethods.get_anti_particle(particle_str=nu_mu_str)  # type: str
    anti_nu_tau_str = LatexObsNameMethods.get_anti_particle(particle_str=nu_tau_str)  # type: str

    star_str = r"$\ast$"  # type: str
    star_star_str = star_str[:-1] + star_str[1:]  # type: str
    w_wo_star_str = r"$(\ast)$"  # type: str

    d_str = r"$\mathrm{D}$"  # type: str
    d_st_str = LatexObsNameMethods.add_super_script(var_str=d_str, script=star_str)  # type: str
    d_st_st_str = LatexObsNameMethods.add_super_script(var_str=d_str, script=star_star_str)  # type: str
    d_w_wo_st_str = LatexObsNameMethods.add_super_script(var_str=d_str, script=w_wo_star_str)  # type: str
    d_w_wo_st_and_pi_str = d_w_wo_st_str[:-1] + pi_str[1:]  # type: str
    d_st_daughters_str = d_st_str[:-1] + r"\;\mathrm{Daughters}$"  # type: str

    d_zero_str = LatexObsNameMethods.add_super_script(var_str=d_str, script=r"$0$")  # type: str
    d_pm_str = LatexObsNameMethods.add_super_script(var_str=d_str, script=r"$\pm$")  # type: str
    d_plus_str = LatexObsNameMethods.add_super_script(var_str=d_str, script=r"$+$")  # type: str
    d_minus_str = LatexObsNameMethods.add_super_script(var_str=d_str, script=r"$-$")  # type: str
    d_st_zero_str = d_st_str[:-1] + r"\!^{0}$"  # type: str
    d_st_pm_str = d_st_str[:-1] + r"\!^{\pm}$"  # type: str
    d_st_plus_str = d_st_str[:-1] + r"\!^{+}$"  # type: str
    d_st_minus_str = d_st_str[:-1] + r"\!^{-}$"  # type: str

    d_sszero_st_str = LatexObsNameMethods.add_sub_script(var_str=d_st_str, script=r"$0$")  # type: str
    d_sszero_st_zero_str = d_sszero_st_str[:-1] + r"\!^{0}$"  # type: str
    d_sszero_st_pm_str = d_sszero_st_str[:-1] + r"\!^{\pm}$"  # type: str
    d_sszero_st_plus_str = d_sszero_st_str[:-1] + r"\!^{+}$"  # type: str
    d_sszero_st_minus_str = d_sszero_st_str[:-1] + r"\!^{-}$"  # type: str

    d_two_st_str = LatexObsNameMethods.add_sub_script(var_str=d_st_str, script=r"$2$")  # type: str
    d_two_st_zero_str = d_two_st_str[:-1] + r"\!^{0}$"  # type: str
    d_two_st_pm_str = d_two_st_str[:-1] + r"\!^{\pm}$"  # type: str
    d_two_st_plus_str = d_two_st_str[:-1] + r"\!^{+}$"  # type: str
    d_two_st_minus_str = d_two_st_str[:-1] + r"\!^{-}$"  # type: str

    d_one_str = LatexObsNameMethods.add_sub_script(var_str=d_str, script=r"$1$")  # type: str
    d_one_zero_str = LatexObsNameMethods.add_super_script(var_str=d_one_str, script=r"$0$")  # type: str
    d_one_pm_str = LatexObsNameMethods.add_super_script(var_str=d_one_str, script=r"$\pm$")  # type: str
    d_one_plus_str = LatexObsNameMethods.add_super_script(var_str=d_one_str, script=r"$+$")  # type: str
    d_one_minus_str = LatexObsNameMethods.add_super_script(var_str=d_one_str, script=r"$-$")  # type: str

    d_prime_one_str = LatexObsNameMethods.add_super_script(var_str=d_one_str, script=r"$\prime$")  # type: str
    d_prime_one_zero_str = d_prime_one_str[:-1] + r"\!^{0}$"  # type: str
    d_prime_one_pm_str = d_prime_one_str[:-1] + r"\!^{\pm}$"  # type: str
    d_prime_one_plus_str = d_prime_one_str[:-1] + r"\!^{+}$"  # type: str
    d_prime_one_minus_str = d_prime_one_str[:-1] + r"\!^{-}$"  # type: str

    d_strange_str = LatexObsNameMethods.add_sub_script(var_str=d_str, script=r"$\mathrm{s}$")  # type: str
    d_strange_zero_str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$0$")  # type: str
    d_strange_pm_str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$\pm$")  # type: str
    d_strange_plus_str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$+$")  # type: str
    d_strange_minus_str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$-$")  # type: str

    d_strange_st_str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=star_str)  # type: str
    d_strange_st_zero_str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$\ast 0$")  # type: str
    d_strange_st_pm_str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$\ast \pm$")  # type: str
    d_strange_st_plus_str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$\ast +$")  # type: str
    d_strange_st_minus_str = LatexObsNameMethods.add_super_script(var_str=d_strange_str, script=r"$\ast -$")  # type: str

    b_str = r"$\mathrm{B}$"  # type: str
    b_zero_str = LatexObsNameMethods.add_super_script(var_str=b_str, script=r"$0$")  # type: str
    b_pm_str = LatexObsNameMethods.add_super_script(var_str=b_str, script=r"$\pm$")  # type: str
    b_plus_str = LatexObsNameMethods.add_super_script(var_str=b_str, script=r"$+$")  # type: str
    b_minus_str = LatexObsNameMethods.add_super_script(var_str=b_str, script=r"$-$")  # type: str

    sig_str = r"$\mathrm{sig}$"  # type: str
    tag_str = r"$\mathrm{tag}$"  # type: str
    b_sig_str = LatexObsNameMethods.add_sub_script(var_str=b_str, script=sig_str)  # type: str
    b_tag_str = LatexObsNameMethods.add_sub_script(var_str=b_str, script=tag_str)  # type: str
    b_sig_minus_str = LatexObsNameMethods.add_super_script(var_str=b_sig_str, script=r"$-$")  # type: str
    b_sig_plus_str = LatexObsNameMethods.add_super_script(var_str=b_sig_str, script=r"$+$")  # type: str
    b_sig_neutral_str = LatexObsNameMethods.add_super_script(var_str=b_sig_str, script=r"$0$")  # type: str
    b_sig_anti_neutral_str = LatexObsNameMethods.get_anti_particle(particle_str=b_sig_neutral_str)  # type: str
    b_tag_minus_str = LatexObsNameMethods.add_super_script(var_str=b_tag_str, script=r"$-$")  # type: str
    b_tag_plus_str = LatexObsNameMethods.add_super_script(var_str=b_tag_str, script=r"$+$")  # type: str
    b_tag_neutral_str = LatexObsNameMethods.add_super_script(var_str=b_tag_str, script=r"$0$")  # type: str
    b_tag_anti_neutral_str = LatexObsNameMethods.get_anti_particle(particle_str=b_tag_neutral_str)  # type: str

    y4s_str = r"$\Upsilon(4\mathrm{S})$"  # type: str

    proton_str = r"$\mathrm{p}$"  # type: str
    proton_pm_str = LatexObsNameMethods.add_super_script(var_str=proton_str, script=r"$\pm$")  # type: str

    xu_str = LatexObsNameMethods.add_sub_script(var_str=r"$\mathrm{X}$", script=r"$\mathrm{u}$")  # type: str
    xu_zero_str = LatexObsNameMethods.add_super_script(var_str=xu_str, script=r"$0$")  # type: str
    xu_pm_str = LatexObsNameMethods.add_super_script(var_str=xu_str, script=r"$\pm$")  # type: str
    xu_plus_str = LatexObsNameMethods.add_super_script(var_str=xu_str, script=r"$+$")  # type: str
    xu_minus_str = LatexObsNameMethods.add_super_script(var_str=xu_str, script=r"$-$")  # type: str

    a_one_str = LatexObsNameMethods.add_sub_script(var_str=r"$\mathrm{a}$", script=r"$1$")  # type: str
    a_one_zero_str = LatexObsNameMethods.add_super_script(var_str=a_one_str, script=r"$0$")  # type: str
    a_one_pm_str = LatexObsNameMethods.add_super_script(var_str=a_one_str, script=r"$\pm$")  # type: str
    a_one_plus_str = LatexObsNameMethods.add_super_script(var_str=a_one_str, script=r"$+$")  # type: str
    a_one_minus_str = LatexObsNameMethods.add_super_script(var_str=a_one_str, script=r"$-$")  # type: str

    eta_str = r"$\eta$"  # type: str
    eta_zero_str = LatexObsNameMethods.add_super_script(var_str=eta_str, script=r"$0$")  # type: str

    # The following strings are just for test purposes:
    __anti_test1_str = LatexObsNameMethods.get_anti_particle(particle_str=d_st_zero_str)  # type: str
    __anti_test2_str = LatexObsNameMethods.get_anti_particle(particle_str=d_st_daughters_str)  # type: str
    __anti_test3_str = LatexObsNameMethods.get_anti_particle(particle_str=muon_pm_str)  # type: str
    __anti_test4_str = LatexObsNameMethods.get_anti_particle(particle_str=y4s_str)  # type: str


if __name__ == "__main__":
    from collections import Counter
    from matplotlib import pyplot as plt

    print("Testing some stuff:")

    all_latex_labels = [label_name for label_name in LatexParticleNames.__dict__.keys() if not label_name.endswith("__")]
    all_particle_labels = [LatexParticleNames.__dict__[label_name] for label_name in all_latex_labels]
    assert len(set(all_particle_labels)) == len(all_particle_labels), (
        len(set(all_particle_labels)),
        len(all_particle_labels),
        [v for v, c in Counter(all_particle_labels).items() if c > 1],
    )

    test_text = ""
    for label_name in all_latex_labels:
        particle_label = LatexParticleNames.__dict__[label_name]
        print("\t- " + label_name + "  ->  " + particle_label)
        assert particle_label.count("{") == particle_label.count("}")
        assert particle_label.count("$") % 2 == 0, particle_label.count("$")

        test_text += f"{particle_label}\n"

        fig, ax = plt.subplots(figsize=(6, 4), tight_layout=True)
        try:
            ax.annotate(text=particle_label, xy=(0.5, 0.5), xytext=(0.6, 0.6))
            plt.show()
        except:  # noqa: E722  # Ignore this flake8 error for this test.
            plt.close(fig)
            fig.clf()
            raise ValueError("Error in " + label_name + "  ->  " + particle_label)

        plt.close(fig)
        fig.clf()

    fig, ax = plt.subplots(figsize=(6, 4), tight_layout=True)
    ax.annotate(text=test_text, xy=(0.5, 0.5), xytext=(0.6, 0.6))
    plt.show()
