"""
Definition of MVA Method for Event Selection Optimization

Patrick Ecker 2020
"""

from typing import Optional, Tuple, List

from rdstar.utilities import PathType

from rdstar.mva_methods.mva_methods import MVABaseClass
from rdstar.mva_methods.mva_validation import MVAValidation
from rdstar.mva_selection.mva_selector_setups import MVASelectorSetups

__all__ = [
    "MVASelector",
    "MVASelectorValidation",
]


class MVASelector(MVABaseClass):
    def __init__(
        self,
        selection_mva_variables: List[str],
        target_variable: str,
        weight: Optional[str] = None,
    ) -> None:
        super().__init__(
            variables=selection_mva_variables,
            target_variable=target_variable,
            weight=weight,
        )


class MVASelectorValidation(MVAValidation):
    default_selector_mva_output_latex_name = "Event Selection MVA"  # type: str

    def __init__(
        self,
        path_train_df: PathType,
        path_test_df: PathType,
        path_mva_var_list: PathType,
        path_mva_bdt_weights: PathType,
        mva_selector_setup_name: str,
        target_dir_path: PathType,
    ) -> None:
        self._mva_setup = MVASelectorSetups.get_selection_mva_setup(setup_name=mva_selector_setup_name)

        super().__init__(
            mva_class=MVASelector,
            path_train_df=path_train_df,
            path_test_df=path_test_df,
            path_var_list=path_mva_var_list,
            path_bdt_weights=path_mva_bdt_weights,
            target_var=self._mva_setup.get_target_column_name(),
            training_histogram_variables_dict=self._mva_setup.get_selection_variable_dict(),
            validation_variables=self._mva_setup.get_validation_variables(),
            mva_output_name=self._mva_setup.get_selection_mva_output_column_name(),
            output_dir_path=target_dir_path,
        )

    def run_validation(
        self,
        base_name: str,
        working_point: float,
        mva_output_latex_name: str,
        dry_run: bool = False,
    ) -> Tuple[List[Tuple[str, PathType]], float, float]:
        return super().run_validation(
            base_name=base_name,
            working_point=working_point,
            mva_output_latex_name=mva_output_latex_name,
            dry_run=dry_run,
        )
