"""
Management and information for analysis plots.

Felix Metzner 2020
"""

import logging

from typing import Dict, Type

from rdstar.offline_analysis.analysis_plots.weight_plotter import WeightPlotter
from rdstar.offline_analysis.analysis_plots.analysis_plot_base import AnalysisPlotBaseClass
from rdstar.offline_analysis.analysis_plots.mc_bin_composition_plotter import MCBinCompositionPlotter
from rdstar.offline_analysis.analysis_plots.q_squared_residual_plotter import QSquaredResidualPlotter

logging.getLogger(__name__).addHandler(logging.NullHandler())

__all__ = [
    "AnalysisPlotManager",
]


class AnalysisPlotManager:
    analysis_plots_base_dir_name = "AnalysisPlots"  # type: str

    analysis_plot_types = {
        QSquaredResidualPlotter.plotter_id_key(): QSquaredResidualPlotter,
        WeightPlotter.plotter_id_key(): WeightPlotter,
        MCBinCompositionPlotter.plotter_id_key(): MCBinCompositionPlotter,
    }  # type: Dict[str, Type[AnalysisPlotBaseClass]]

    def __init__(self) -> None:
        pass

    @staticmethod
    def get_plotter_class(plot_type_key: str) -> Type[AnalysisPlotBaseClass]:
        assert plot_type_key in AnalysisPlotManager.analysis_plot_types.keys(), (
            plot_type_key,
            AnalysisPlotManager.analysis_plot_types.keys(),
        )
        return AnalysisPlotManager.analysis_plot_types[plot_type_key]
