from typing import List, Dict
import pandas as pd

from rdstar.offline_analysis.mc_ids import MatchQueryInfo, SigQueryDictType
from rdstar.offline_analysis.efficiency_calculation.simple_efficiency_calculator import DataTypeFraction

from templatefitter.plotter.plot_style import KITColors


sig_dss_components = {
    "ChargedBDzeroStarPiZ": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=201),
    "ChargedBDonePrimePiZ": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=202),
    "ChargedBDonePiZ": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=203),
    "ChargedBDtwoStarPiZ": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=204),
    "ChargedBDzeroStarPiP": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=205),
    "ChargedBDonePrimePiP": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=206),
    "ChargedBDonePiP": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=207),
    "ChargedBDtwoStarPiP": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=208),
    "ChargedBGapDone1": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=209),
    "ChargedBGapDone2": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=210),
    "ChargedBDl": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=211),
    "ChargedBDSl": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=212),
    "ChargedBBBarBkg": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=213),
    "ChargedBBKG": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=214),
    "NeutralBDzeroStarPiZ": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=301),
    "NeutralBDonePrimePiZ": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=302),
    "NeutralBDonePiZ": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=303),
    "NeutralBDtwoStarPiZ": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=304),
    "NeutralBDzeroStarPiP": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=305),
    "NeutralBDonePrimePiP": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=306),
    "NeutralBDonePiP": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=307),
    "NeutralBDtwoStarPiP": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=308),
    "NeutralBGapDone1": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=309),
    "NeutralBGapDone2": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=310),
    "NeutralBDl": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=311),
    "NeutralBDSl": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=312),
    "NeutralBBBarBkg": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=313),
    "NeutralBBKG": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=314),
}  # type: SigQueryDictType

sig_dss_labels = {
    "BKG": "BKG",
    "NormBzeroDs": r"$B^0 \rightarrow D^{\ast} \ell \nu $ Norm",
    "DstarstarPiZ": r"$B \rightarrow D^{**}(\rightarrow D^{(\ast)} \pi^0) \ell \nu$ BKG",
    "ChargedBDzeroStarPiP": r"$B^+ \rightarrow D_0^{\ast} (\rightarrow D \pi^\pm) \ell \nu $",
    "ChargedBDonePrimePiP": r"$B^+ \rightarrow D_1^\prime (\rightarrow D^\ast \pi^\pm) \ell \nu $",
    "ChargedBDonePiP": r"$B^+ \rightarrow D_1 (\rightarrow D^\ast \pi^\pm) \ell \nu $",
    "ChargedBDtwoStarPiP": r"$B^+ \rightarrow D_2^\ast (\rightarrow D^{(\ast)} \pi^\pm) \ell \nu $",
    "NormBchargedDs": r"$B^+ \rightarrow D^{\ast} \ell \nu $ Norm",
    "NeutralBDzeroStarPiP": r"$B^0 \rightarrow D_0^{\ast} (\rightarrow D \pi^\pm) \ell \nu $",
    "NeutralBDonePrimePiP": r"$B^0 \rightarrow D_1^\prime (\rightarrow D^\ast \pi^\pm) \ell \nu $",
    "NeutralBDonePiP": r"$B^0 \rightarrow D_1 (\rightarrow D^\ast \pi^\pm) \ell \nu $",
    "NeutralBDtwoStarPiP": r"$B^0 \rightarrow D_2^\ast (\rightarrow D^{(\ast)} \pi^\pm) \ell \nu $",
    "DstarstarPiP": r"$B \rightarrow D^{**}(\rightarrow D^{(\ast)} \pi^\pm) \ell \nu$ BKG",
    "ChargedBDzeroStarPiZ": r"$B^+ \rightarrow D_0^{\ast} (\rightarrow D \pi^0) \ell \nu $",
    "ChargedBDonePrimePiZ": r"$B^+ \rightarrow D_1^\prime (\rightarrow D^\ast \pi^0) \ell \nu $",
    "ChargedBDonePiZ": r"$B^+ \rightarrow D_1 (\rightarrow D^\ast \pi^0) \ell \nu $",
    "ChargedBDtwoStarPiZ": r"$B^+ \rightarrow D_2^\ast (\rightarrow D^{(\ast)} \pi^0) \ell \nu $",
    "NeutralBDzeroStarPiZ": r"$B^0 \rightarrow D_0^{\ast} (\rightarrow D \pi^0) \ell \nu $",
    "NeutralBDonePrimePiZ": r"$B^0 \rightarrow D_1^\prime (\rightarrow D^\ast \pi^0) \ell \nu $",
    "NeutralBDonePiZ": r"$B^0 \rightarrow D_1 (\rightarrow D^\ast \pi^0) \ell \nu $",
    "NeutralBDtwoStarPiZ": r"$B^0 \rightarrow D_2^\ast (\rightarrow D^{(\ast)} \pi^0) \ell \nu $",
}  # type: Dict[str, str]

sig_dss_colors = {
    "BKG": "white",
    "NormBzeroDs": KITColors.kit_cyan,
    "DstarstarPiZ": KITColors.light_grey,
    "ChargedBDzeroStarPiP": KITColors.kit_purple,
    "ChargedBDonePrimePiP": KITColors.kit_orange,
    "ChargedBDonePiP": KITColors.kit_green,
    "ChargedBDtwoStarPiP": KITColors.kit_maygreen,
    "NormBchargedDs": KITColors.kit_cyan,
    "NeutralBDzeroStarPiP": KITColors.kit_purple,
    "NeutralBDonePrimePiP": KITColors.kit_orange,
    "NeutralBDonePiP": KITColors.kit_green,
    "NeutralBDtwoStarPiP": KITColors.kit_maygreen,
    "DstarstarPiP": KITColors.light_grey,
    "ChargedBDzeroStarPiZ": KITColors.kit_purple,
    "ChargedBDonePrimePiZ": KITColors.kit_orange,
    "ChargedBDonePiZ": KITColors.kit_green,
    "ChargedBDtwoStarPiZ": KITColors.kit_maygreen,
    "NeutralBDzeroStarPiZ": KITColors.kit_purple,
    "NeutralBDonePrimePiZ": KITColors.kit_orange,
    "NeutralBDonePiZ": KITColors.kit_green,
    "NeutralBDtwoStarPiZ": KITColors.kit_maygreen,
}  # type: Dict[str, str]

BZERO_TO_DSTAR_E_BF = 0.0533
BZERO_TO_DZEROSTAR_E_BF = 0.0084
BZERO_TO_DONE_E_BF = 0.0074
BZERO_TO_DONEPRIME_E_BF = 0.0074
BZERO_TO_DTWOSTAR_E_BF = 0.0036

BPLUS_TO_DSTAR_E_BF = 0.0579
BPLUS_TO_DZEROSTAR_E_BF = 0.0091
BPLUS_TO_DONE_E_BF = 0.0081
BPLUS_TO_DONEPRIME_E_BF = 0.0081
BPLUS_TO_DTWOSTAR_E_BF = 0.0039

sig_dss_nominal_bfs = {
    "ChargedBDzeroStarPiP": 0.00389 * 2 * 2 / 3,
    "ChargedBDonePrimePiP": 0.00431 * 2 * 2 / 3,
    "ChargedBDonePiP": 0.00454 * 2 * 2 / 3,
    "ChargedBDtwoStarPiP": 0.00373 * 2 * 2 / 3,
    "NeutralBDzeroStarPiP": 0.00362 * 2 * 2 / 3,
    "NeutralBDonePrimePiP": 0.00401 * 2 * 2 / 3,
    "NeutralBDonePiP": 0.00422 * 2 * 2 / 3,
    "NeutralBDtwoStarPiP": 0.00347 * 2 * 2 / 3,
    "ChargedBDzeroStarPiZ": 0.00389 * 2 * 1 / 3,
    "ChargedBDonePrimePiZ": 0.00431 * 2 * 1 / 3,
    "ChargedBDonePiZ": 0.00454 * 2 * 1 / 3,
    "ChargedBDtwoStarPiZ": 0.00373 * 2 * 1 / 3,
    "NeutralBDzeroStarPiZ": 0.00362 * 2 * 1 / 3,
    "NeutralBDonePrimePiZ": 0.00401 * 2 * 1 / 3,
    "NeutralBDonePiZ": 0.00422 * 2 * 1 / 3,
    "NeutralBDtwoStarPiZ": 0.00347 * 2 * 1 / 3,
}  # type: Dict[str, float]

charged_B_charged_pi_sig_dss_plot_order = {
    "BKG": [
        "ChargedBBKG",
        "ChargedBBBarBkg",
        "ChargedBDl",
        "ChargedBGapDone1",
        "ChargedBGapDone2",
        "NeutralBBKG",
        "NeutralBBBarBkg",
        "NeutralBDl",
        "NeutralBGapDone1",
        "NeutralBGapDone2",
        "ChargedBDSl",
        "NeutralBDzeroStarPiP",
        "NeutralBDonePrimePiP",
        "NeutralBDtwoStarPiP",
        "NeutralBDonePiP",
    ],
    "NormBzeroDs": [
        "NeutralBDSl",
    ],
    "DstarstarPiZ": [
        "ChargedBDzeroStarPiZ",
        "ChargedBDonePrimePiZ",
        "ChargedBDtwoStarPiZ",
        "ChargedBDonePiZ",
        "NeutralBDzeroStarPiZ",
        "NeutralBDonePrimePiZ",
        "NeutralBDtwoStarPiZ",
        "NeutralBDonePiZ",
    ],
    "ChargedBDzeroStarPiP": [
        "ChargedBDzeroStarPiP",
    ],
    "ChargedBDonePrimePiP": [
        "ChargedBDonePrimePiP",
    ],
    "ChargedBDtwoStarPiP": [
        "ChargedBDtwoStarPiP",
    ],
    "ChargedBDonePiP": [
        "ChargedBDonePiP",
    ],
}  # type: Dict[str, List[str]]

charged_B_charged_pi_sig_dss_data_type_mapping = {
    sig_dss_components["NeutralBDSl"].id: DataTypeFraction(
        data_types=("Mixed",),
        fraction=2 * 2 * BZERO_TO_DSTAR_E_BF,
    ),
    sig_dss_components["ChargedBDzeroStarPiP"].id: DataTypeFraction(
        data_types=("BMD0sE", "BMD0sMu"),
        fraction=(1 + BPLUS_TO_DZEROSTAR_E_BF) * 2 / 3,
    ),
    sig_dss_components["ChargedBDonePrimePiP"].id: DataTypeFraction(
        data_types=("BMDp1E", "BMDp1Mu"),
        fraction=(1 + BPLUS_TO_DONEPRIME_E_BF) * 2 / 3,
    ),
    sig_dss_components["ChargedBDonePiP"].id: DataTypeFraction(
        data_types=("BMD1E", "BMD1Mu"),
        fraction=(1 + BPLUS_TO_DONE_E_BF) * 2 / 3,
    ),
    sig_dss_components["ChargedBDtwoStarPiP"].id: DataTypeFraction(
        data_types=("BMD2sE", "BMD2sMu"),
        fraction=(1 + BPLUS_TO_DTWOSTAR_E_BF) * 2 / 3,
    ),
}  # type: Dict[int, DataTypeFraction]

neutral_B_charged_pi_sig_dss_plot_order = {
    "BKG": [
        "ChargedBBKG",
        "ChargedBBBarBkg",
        "ChargedBDl",
        "ChargedBGapDone1",
        "ChargedBGapDone2",
        "NeutralBBKG",
        "NeutralBBBarBkg",
        "NeutralBDl",
        "NeutralBGapDone1",
        "NeutralBGapDone2",
        "NeutralBDSl",
        "ChargedBDzeroStarPiP",
        "ChargedBDonePrimePiP",
        "ChargedBDtwoStarPiP",
        "ChargedBDonePiP",
    ],
    "NormBchargedDs": [
        "ChargedBDSl",
    ],
    "DstarstarPiZ": [
        "ChargedBDzeroStarPiZ",
        "ChargedBDonePrimePiZ",
        "ChargedBDtwoStarPiZ",
        "ChargedBDonePiZ",
        "NeutralBDzeroStarPiZ",
        "NeutralBDonePrimePiZ",
        "NeutralBDtwoStarPiZ",
        "NeutralBDonePiZ",
    ],
    "NeutralBDzeroStarPiP": [
        "NeutralBDzeroStarPiP",
    ],
    "NeutralBDonePrimePiP": [
        "NeutralBDonePrimePiP",
    ],
    "NeutralBDtwoStarPiP": [
        "NeutralBDtwoStarPiP",
    ],
    "NeutralBDonePiP": [
        "NeutralBDonePiP",
    ],
}  # type: Dict[str, List[str]]

neutral_B_charged_pi_sig_dss_data_type_mapping = {
    sig_dss_components["ChargedBDSl"].id: DataTypeFraction(
        data_types=("Charged",),
        fraction=2 * 2 * BPLUS_TO_DSTAR_E_BF,
    ),
    sig_dss_components["NeutralBDzeroStarPiP"].id: DataTypeFraction(
        data_types=("BZD0sE", "BZD0sMu"),
        fraction=(1 + BZERO_TO_DZEROSTAR_E_BF) * 2 / 3,
    ),
    sig_dss_components["NeutralBDonePrimePiP"].id: DataTypeFraction(
        data_types=("BZDp1E", "BZDp1Mu"),
        fraction=(1 + BZERO_TO_DONEPRIME_E_BF) * 2 / 3,
    ),
    sig_dss_components["NeutralBDonePiP"].id: DataTypeFraction(
        data_types=("BZD1E", "BZD1Mu"),
        fraction=(1 + BZERO_TO_DONE_E_BF) * 2 / 3,
    ),
    sig_dss_components["NeutralBDtwoStarPiP"].id: DataTypeFraction(
        data_types=("BZD2sE", "BZD2sMu"),
        fraction=(1 + BZERO_TO_DTWOSTAR_E_BF) * 2 / 3,
    ),
}  # type: Dict[int, DataTypeFraction]

charged_B_neutral_pi_sig_dss_plot_order = {
    "BKG": [
        "ChargedBBKG",
        "ChargedBBBarBkg",
        "ChargedBDl",
        "ChargedBGapDone1",
        "ChargedBGapDone2",
        "NeutralBBKG",
        "NeutralBBBarBkg",
        "NeutralBDl",
        "NeutralBGapDone1",
        "NeutralBGapDone2",
        "NeutralBDSl",
        "NeutralBDzeroStarPiZ",
        "NeutralBDonePrimePiZ",
        "NeutralBDtwoStarPiZ",
        "NeutralBDonePiZ",
    ],
    "NormBchargedDs": [
        "ChargedBDSl",
    ],
    "DstarstarPiP": [
        "ChargedBDzeroStarPiP",
        "ChargedBDonePrimePiP",
        "ChargedBDtwoStarPiP",
        "ChargedBDonePiP",
        "NeutralBDzeroStarPiP",
        "NeutralBDonePrimePiP",
        "NeutralBDtwoStarPiP",
        "NeutralBDonePiP",
    ],
    "ChargedBDzeroStarPiZ": [
        "ChargedBDzeroStarPiZ",
    ],
    "ChargedBDonePrimePiZ": [
        "ChargedBDonePrimePiZ",
    ],
    "ChargedBDtwoStarPiZ": [
        "ChargedBDtwoStarPiZ",
    ],
    "ChargedBDonePiZ": [
        "ChargedBDonePiZ",
    ],
}  # type: Dict[str, List[str]]

charged_B_neutral_pi_sig_dss_data_type_mapping = {
    sig_dss_components["ChargedBDSl"].id: DataTypeFraction(
        data_types=("Charged",),
        fraction=2 * 2 * BPLUS_TO_DSTAR_E_BF,
    ),
    sig_dss_components["ChargedBDzeroStarPiZ"].id: DataTypeFraction(
        data_types=("BMD0sE", "BMD0sMu"),
        fraction=(1 + BPLUS_TO_DZEROSTAR_E_BF) * 1 / 3,
    ),
    sig_dss_components["ChargedBDonePrimePiZ"].id: DataTypeFraction(
        data_types=("BMDp1E", "BMDp1Mu"),
        fraction=(1 + BPLUS_TO_DONEPRIME_E_BF) * 1 / 3,
    ),
    sig_dss_components["ChargedBDonePiZ"].id: DataTypeFraction(
        data_types=("BMD1E", "BMD1Mu"),
        fraction=(1 + BPLUS_TO_DONE_E_BF) * 1 / 3,
    ),
    sig_dss_components["ChargedBDtwoStarPiZ"].id: DataTypeFraction(
        data_types=("BMD2sE", "BMD2sMu"),
        fraction=(1 + BPLUS_TO_DTWOSTAR_E_BF) * 1 / 3,
    ),
}  # type: Dict[int, DataTypeFraction]

neutral_B_neutral_pi_sig_dss_plot_order = {
    "BKG": [
        "ChargedBBKG",
        "ChargedBBBarBkg",
        "ChargedBDl",
        "ChargedBGapDone1",
        "ChargedBGapDone2",
        "NeutralBBKG",
        "NeutralBBBarBkg",
        "NeutralBDl",
        "NeutralBGapDone1",
        "NeutralBGapDone2",
        "ChargedBDSl",
        "ChargedBDzeroStarPiZ",
        "ChargedBDonePrimePiZ",
        "ChargedBDtwoStarPiZ",
        "ChargedBDonePiZ",
    ],
    "NormBzeroDs": ["NeutralBDSl"],
    "DstarstarPiP": [
        "ChargedBDzeroStarPiP",
        "ChargedBDonePrimePiP",
        "ChargedBDtwoStarPiP",
        "ChargedBDonePiP",
        "NeutralBDzeroStarPiP",
        "NeutralBDonePrimePiP",
        "NeutralBDtwoStarPiP",
        "NeutralBDonePiP",
    ],
    "NeutralBDzeroStarPiZ": [
        "NeutralBDzeroStarPiZ",
    ],
    "NeutralBDonePrimePiZ": [
        "NeutralBDonePrimePiZ",
    ],
    "NeutralBDtwoStarPiZ": [
        "NeutralBDtwoStarPiZ",
    ],
    "NeutralBDonePiZ": [
        "NeutralBDonePiZ",
    ],
}  # type: Dict[str, List[str]]

neutral_B_neutral_pi_sig_dss_data_type_mapping = {
    sig_dss_components["NeutralBDSl"].id: DataTypeFraction(
        data_types=("Mixed",),
        fraction=2 * 2 * BZERO_TO_DSTAR_E_BF,
    ),
    sig_dss_components["NeutralBDzeroStarPiZ"].id: DataTypeFraction(
        data_types=("BZD0sE", "BZD0sMu"),
        fraction=(1 + BZERO_TO_DZEROSTAR_E_BF) * 1 / 3,
    ),
    sig_dss_components["NeutralBDonePrimePiZ"].id: DataTypeFraction(
        data_types=("BZDp1E", "BZDp1Mu"),
        fraction=(1 + BZERO_TO_DONEPRIME_E_BF) * 1 / 3,
    ),
    sig_dss_components["NeutralBDonePiZ"].id: DataTypeFraction(
        data_types=("BZD1E", "BZD1Mu"),
        fraction=(1 + BZERO_TO_DONE_E_BF) * 1 / 3,
    ),
    sig_dss_components["NeutralBDtwoStarPiZ"].id: DataTypeFraction(
        data_types=("BZD2sE", "BZD2sMu"),
        fraction=(1 + BZERO_TO_DTWOSTAR_E_BF) * 1 / 3,
    ),
}  # type: Dict[int, DataTypeFraction]
