from typing import List, Dict
import pandas as pd

from rdstar.offline_analysis.mc_ids import MatchQueryInfo, SigQueryDictType

from templatefitter.plotter.plot_style import KITColors


advanced_dss_components = {
    "DPiZ": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=101),
    "DSPiZ": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=102),
    "DPiP": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=103),
    "DSPiP": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=104),
    "Dl": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=105),
    "DSl": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=106),
    "BKG": MatchQueryInfo(query=lambda df: pd.Series(data=True, index=df.index), id=107),
}  # type: SigQueryDictType

reduced_dss_labels = {
    "BKG": "BKG",
    "Dl": r"$D^{(*)} \ell \nu$",
    "DSSPiP": r"$D^{**}(\rightarrow D^{(*)} \pi^\pm) \ell \nu$",
    "DSPiZ": r"$D^{**}(\rightarrow D^{*} \pi^0) \ell \nu$",
    "DPiZ": r"$D^{**}(\rightarrow D \pi^0) \ell \nu$",
    "DSSPiZ": r"$D^{**}(\rightarrow D^{(*)} \pi^0) \ell \nu$",
    "DSPiP": r"$D^{**}(\rightarrow D^{*} \pi^\pm) \ell \nu$",
    "DPiP": r"$D^{**}(\rightarrow D \pi^\pm) \ell \nu$",
}  # type: Dict[str, str]

reduced_dss_colors = {
    "BKG": KITColors.light_grey,
    "Dl": KITColors.kit_purple,
    "DSSPiP": KITColors.kit_orange,
    "DSPiZ": KITColors.kit_maygreen,
    "DPiZ": KITColors.kit_green,
    "DSSPiZ": KITColors.kit_red,
    "DSPiP": KITColors.kit_cyan,
    "DPiP": KITColors.kit_blue,
}  # type: Dict[str, str]

reduced_dss_mapping = {
    "BKG": ["BKG"],
    "Dl": ["Dl", "DSl"],
    "DSSPiP": ["DPiP", "DSPiP"],
    "DSPiZ": ["DSPiZ"],
    "DPiZ": ["DPiZ"],
    "DSSPiZ": ["DPiZ", "DSPiZ"],
    "DSPiP": ["DSPiP"],
    "DPiP": ["DPiP"],
}  # type: Dict[str, List[str]]

reduced_advanced_dss_plot_order_for_DPiZ = {
    "BKG": ["BKG"],
    "Dl": ["Dl", "DSl"],
    "DSSPiP": ["DPiP", "DSPiP"],
    "DSPiZ": ["DSPiZ"],
    "DPiZ": ["DPiZ"],
}  # type: Dict[str, List[str]]

reduced_advanced_dss_plot_order_for_DSPiZ = {
    "BKG": ["BKG"],
    "Dl": ["Dl", "DSl"],
    "DSSPiP": ["DPiP", "DSPiP"],
    "DPiZ": ["DPiZ"],
    "DSPiZ": ["DSPiZ"],
}  # type: Dict[str, List[str]]

reduced_advanced_dss_plot_order_for_DPiP = {
    "BKG": ["BKG"],
    "Dl": ["Dl", "DSl"],
    "DSSPiZ": ["DPiZ", "DSPiZ"],
    "DSPiP": ["DSPiP"],
    "DPiP": ["DPiP"],
}  # type: Dict[str, List[str]]

reduced_advanced_dss_plot_order_for_DSPiP = {
    "BKG": ["BKG"],
    "Dl": ["Dl", "DSl"],
    "DSSPiZ": ["DPiZ", "DSPiZ"],
    "DPiP": ["DPiP"],
    "DSPiP": ["DSPiP"],
}  # type: Dict[str, List[str]]
