import b2luigi as luigi

from rdstar.offline_analysis.dstst_analysis.utility import *
from rdstar.offline_analysis.dstst_analysis.template_definitions import *


class CombinedDataSampleTask(luigi.ExternalTask):

    def output(self):
        source_dir = "/storage/9/fmetzner/rdstar/data_set_prod_18thApr2019_stream0/CombinedSamples"
        targets = {
            "dss_df": luigi.LocalTarget(os.path.join(source_dir, "Dss_first_selection_with_CS_CS1.h5")),
            "gap_df": luigi.LocalTarget(os.path.join(source_dir, "Gap_first_selection_with_CS_CS1.h5")),
            "mixed_df": luigi.LocalTarget(os.path.join(source_dir, "Charged_first_selection_with_CS_CS1.h5")),
            "charged_df": luigi.LocalTarget(os.path.join(source_dir, "Mixed_first_selection_with_CS_CS1.h5")),
            "charm_df": luigi.LocalTarget(os.path.join(source_dir, "Charm_first_selection_with_CS_CS1.h5")),
            "uds_df": luigi.LocalTarget(os.path.join(source_dir, "UDS_first_selection_with_CS_CS1.h5")),
        }

        return targets


class CombinedSidebandDataSampleTask(luigi.ExternalTask):

    def output(self):
        source_dir = "/storage/9/fmetzner/rdstar/data_set_prod_18thApr2019_stream0/CombinedSamples"
        targets = {
            "dss_df": luigi.LocalTarget(os.path.join(source_dir, "Dss_first_selection_with_CS_CS1_mbc_sideband.h5")),
            "gap_df": luigi.LocalTarget(os.path.join(source_dir, "Gap_first_selection_with_CS_CS1_mbc_sideband.h5")),
            "mixed_df": luigi.LocalTarget(
                os.path.join(source_dir, "Charged_first_selection_with_CS_CS1_mbc_sideband.h5")),
            "charged_df": luigi.LocalTarget(
                os.path.join(source_dir, "Mixed_first_selection_with_CS_CS1_mbc_sideband.h5")),
            "charm_df": luigi.LocalTarget(
                os.path.join(source_dir, "Charm_first_selection_with_CS_CS1_mbc_sideband.h5")),
            "uds_df": luigi.LocalTarget(os.path.join(source_dir, "UDS_first_selection_with_CS_CS1_mbc_sideband.h5")),
        }

        return targets


@luigi.requires(CombinedDataSampleTask)
class PrepareDataSampleTask(luigi.Task):

    def run(self):
        dataframes = {key: pd.read_hdf(value.path) for key, value in self.input()[0].items()}
        total_mc = combine_dataframe_dict(dataframes)
        total_mc = prepare_weights(total_mc)
        reduced_dataset = total_mc.drop(columns=drop_list)
        reduced_dataset.to_hdf(self.get_output_file_name("fit_dataset.h5"), key="fit")

    def output(self):
        yield self.add_to_output("fit_dataset.h5")


@luigi.requires(CombinedSidebandDataSampleTask)
class PrepareSidebandDataSampleTask(luigi.Task):

    def run(self):
        dataframes = {key: pd.read_hdf(value.path) for key, value in self.input()[0].items()}
        total_mc = combine_dataframe_dict(dataframes)
        total_mc = prepare_weights(total_mc)
        reduced_dataset = total_mc.drop(columns=drop_list)
        reduced_dataset.to_hdf(self.get_output_file_name("fit_sideband_dataset.h5"), key="fit")

    def output(self):
        yield self.add_to_output("fit_sideband_dataset.h5")


@luigi.requires(PrepareDataSampleTask)
class TemplatePlotsTask(luigi.Task):

    def run(self):
        pass

    def output(self):
        pass


class FitMasterTask(luigi.WrapperTask):

    def requires(self):
        yield PrepareDataSampleTask()
        yield PrepareSidebandDataSampleTask()


if __name__ == "__main__":
    luigi.process(FitMasterTask())
