import os
import yaml
import click

import rdstar

from analysistools.batch_tools.file_handler import load_json_config
from analysistools.dataframe_tools.dataframe_tools import copy_online_analysis_output_from_remote

from rdstar.utilities import PathType


@click.command()
@click.option(
    "--config",
    "-c",
    "config_file",
    required=True,
    type=click.Path(exists=True),
    help="path to offline analysis config",
)
def copy_files_from_remote(config_file):
    """
    This script copies the files specified in the offline config from the kekcc.
    """
    with open(config_file) as f:
        config = yaml.load(f, Loader=yaml.FullLoader)

    pkg_dir = os.path.dirname(rdstar.__file__)  # type: PathType
    prod_tag = config["production_tag"]

    batch_configs = [load_json_config(os.path.join(pkg_dir, "configs", prod_tag, c)) for c in config["batch_configs"]]

    target_dir = config["working_dir"]

    remote_host = config["file_transfer"]["remote_host"]
    remote_src_dir = config["file_transfer"]["remote_source_dir"]

    copy_online_analysis_output_from_remote(
        remote_host=remote_host,
        source_dir=remote_src_dir,
        target_dir=target_dir,
        configs=batch_configs,
    )
