"""
Base class for offline observable definition.

Felix Metzner 2021
"""

import logging
import numpy as np
import pandas as pd

from abc import ABC, abstractmethod
from typing import Union, Optional, Tuple, Dict

from templatefitter.binned_distributions.binning import Binning
from templatefitter.plotter.histogram_variable import HistVariable


logging.getLogger(__name__).addHandler(logging.NullHandler())


__all__ = [
    "BinnedOfflineObservableBase",
]


class BinnedOfflineObservableBase(ABC):
    new_column_name = ""  # type: str
    required_variables = {}  # type: Dict[str, str]

    hist_var = None  # type: Optional[HistVariable]
    binning = None  # type: Optional[Binning]

    @classmethod
    @abstractmethod
    def calculate_new_column_values(cls, df: pd.DataFrame, check_range: bool = True) -> Union[pd.Series, np.array]:
        raise NotImplementedError

    @classmethod
    def get_new_column_name(cls) -> str:
        return cls.new_column_name

    @classmethod
    def get_required_columns(cls) -> Tuple[str, ...]:
        return tuple(cls.required_variables.values())

    @classmethod
    def get_histogram_variable_info(cls) -> Optional[Tuple[HistVariable, Binning]]:
        if cls.hist_var is None:
            return None
        return cls.hist_var, cls.binning

    @staticmethod
    def get_tick_label_mapping() -> Optional[Dict[int, str]]:
        return None
