"""
Continuum Suppression MVA Validation

Felix Metzner 2020
"""

from typing import Tuple, List

from rdstar.utilities import PathType
from rdstar.mva_methods.mva_validation import MVAValidation
from rdstar.continuum_suppression.continuum_suppression import (
    ContinuumSuppression,
    get_cs_variable_dict,
    cs_validation_variables,
)

__all__ = [
    "CSValidation",
]


class CSValidation(MVAValidation):
    default_cs_mva_output_latex_name = "Continuum Suppression"  # type: str

    def __init__(
        self,
        path_train_df: PathType,
        path_test_df: PathType,
        path_cs_var_list: PathType,
        path_cs_bdt_weights: PathType,
        cs_output_col_name: str,
        cs_target_var: str,
        target_dir_path: PathType,
        using_sig_side_vars: bool = True,
    ) -> None:
        cs_hist_variable_dict = get_cs_variable_dict(signal_side=using_sig_side_vars)

        super().__init__(
            mva_class=ContinuumSuppression,
            path_train_df=path_train_df,
            path_test_df=path_test_df,
            path_var_list=path_cs_var_list,
            path_bdt_weights=path_cs_bdt_weights,
            target_var=cs_target_var,
            training_histogram_variables_dict=cs_hist_variable_dict,
            validation_variables=cs_validation_variables,
            mva_output_name=cs_output_col_name,
            output_dir_path=target_dir_path,
        )

    def run_validation(
        self,
        base_name: str,
        working_point: float,
        mva_output_latex_name: str,
        dry_run: bool = False,
    ) -> Tuple[List[Tuple[str, PathType]], float, float]:
        return super().run_validation(
            base_name=base_name,
            working_point=working_point,
            mva_output_latex_name=mva_output_latex_name,
            dry_run=dry_run,
        )
