#!/bin/bash

if [ $# -eq 0 ]
then
  echo 'Usage: '"$0"' /path/to/installation/location/ (directory must already exist!)'
  exit
fi

INSTALLDIRPATH="$1"

[ ! -d "$INSTALLDIRPATH" ] && echo "The installation directory $INSTALLDIRPATH does not exists! Please create it! Aborting" && exit 1
ERRORINFOTEXTONE="The the RDStar repository is not available under $INSTALLDIRPATH/RDStar!"
ERRORINFOTEXTTWO="Please clone it into the installation repository! Aborting"
[ ! -d "$INSTALLDIRPATH"/RDStar ] && echo "$ERRORINFOTEXTONE $ERRORINFOTEXTTWO" && exit 1

echo "Installing setup into $INSTALLDIRPATH"

cd "$INSTALLDIRPATH" || exit

if [[ ! -d $INSTALLDIRPATH/setup_basf2_at_kek.sh ]]
then
  echo "Creating setup script in $INSTALLDIRPATH/setup_basf2_at_kek.sh"
  cp ./RDStar/scripts/template_setup_basf2_at_kek.sh ./setup_basf2_at_kek.sh
  sed -i 's@PATHTOSETUPLOCATION@'"$INSTALLDIRPATH"'@g' ./setup_basf2_at_kek.sh
  chmod u+x ./setup_basf2_at_kek.sh
fi

if [[ ! -d $INSTALLDIRPATH/basf2 ]]
then
  echo "Setting up basf2"
  mkdir basf2
  cd ./basf2 || exit
  git clone ssh://git@stash.desy.de:7999/b2/tools.git
  source ./tools/b2setup
  mkdir externals
  b2install-externals v01-06-01 el7
  b2code-create basf2
  cd ./basf2 || exit
  git checkout feature/analysis_branch_b2bii_rdstar
  b2setup
  b2code-option-externals opt
  b2code-option opt
  scons
  cd ../../
else
  echo "Updating basf2"
  cd ./basf2 || exit
  source ./tools/b2setup
  cd ./basf2 || exit
  b2setup
  git checkout feature/analysis_branch_b2bii_rdstar
  git pull
  b2code-option-externals opt
  b2code-option opt
  scons
  cd ../../
fi
echo "basf2 setup completed"

echo "Upgrading pip"
pip3 install --upgrade pip

echo "Updating RDStar"
cd ./RDStar || exit
git pull
cd ../

echo "Preparing scripts and local folders"
[ ! -d "$INSTALLDIRPATH"/databases ] && mkdir -p databases/RDStar_b2bii/mc_beamparams/localdb

if [[ ! -d $INSTALLDIRPATH/cleanup.py ]]
then
  cp ./RDStar/scripts/cleanup.py ./cleanup.py
fi
if [[ ! -d $INSTALLDIRPATH/cleanup_low_level_study.py ]]
then
  cp ./RDStar/scripts/cleanup_low_level_study.py ./cleanup_low_level_study.py
fi

echo "Cloning relevant repositories"
if [[ ! -d $INSTALLDIRPATH/AnalysisTools ]]
then
  git clone git@gitlab.etp.kit.edu:fmetzner/AnalysisTools.git
else
  cd AnalysisTools || exit
  git checkout master
  git pull
  cd ../
fi
if [[ ! -d $INSTALLDIRPATH/TemplateFitter ]]
then
  git clone git@github.com:FelixMetzner/TemplateFitter.git
else
  cd TemplateFitter || exit
  git checkout master
  git pull
  cd ../
fi
if [[ ! -d $INSTALLDIRPATH/eFFORT ]]
then
  git clone git@gitlab.etp.kit.edu:fmetzner/eFFORT.git
else
  cd eFFORT || exit
  git checkout master
  git pull
  cd ../
fi
if [[ ! -d $INSTALLDIRPATH/probfit ]]
then
  git clone git@github.com:scikit-hep/probfit.git
else
  cd probfit || exit
  git checkout master
  git pull
  cd ../
fi
if [[ ! -d $INSTALLDIRPATH/b2luigi ]]
then
  git clone https://github.com/nils-braun/b2luigi
else
  cd b2luigi || exit
  git checkout main
  git pull
  cd ../
fi

echo "Installing b2luigi"
cd ./b2luigi || exit
pip3 uninstall -y b2luigi
pip3 install flit
flit install -s
cd ../

echo "Installing probfit"
pip3 install cython
cd ./probfit || exit
cp ../RDStar/basf2_patches/probfit_iminuit_2.0_fix.diff ./
git apply probfit_iminuit_2.0_fix.diff
rm probfit_iminuit_2.0_fix.diff
pip3 install -e ./
cd ../

echo "Installing eFFORT"
cd ./eFFORT || exit
pip3 install -e ./
cd ../

echo "Installing TemplateFitter"
cd ./TemplateFitter || exit
pip3 install -r requirements.txt
pip3 install -e ./
cd ../

echo "Installing AnalysisTools"
cd ./AnalysisTools/ || exit
pip3 install -r requirements.txt
pip3 install -e ./
cd ../

echo "Installing RDStar"
cd ./RDStar/ || exit
sed -i "s/git+ssh/#git+ssh/g" ./requirements.txt
pip3 install -r requirements.txt
pip3 install -e ./
sed -i "s/#git+ssh/git+ssh/g" ./requirements.txt
cd ../

echo "Reinstalling in correct order!"
cd ./b2luigi || exit
pip3 uninstall -y b2luigi
flit install -s
cd ../probfit || exit
pip3 install -e ./
cd ../TemplateFitter || exit
pip3 install -e ./
cd ../eFFORT || exit
pip3 install -e ./
cd ../AnalysisTools/ || exit
pip3 install -e ./
cd ../RDStar/ || exit
pip3 install -e ./
echo "Setup completed! :)"
